/** @package disp.c
 * @brief Dette modul står for at åbne SPI forbindelsen til displayet, sende de maximum 4 HEX cifre og lukke forbindelsen.
 *
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>


#include <fcntl.h>
#include "disp.h"

#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
#define ERROR -1

/** @brief Denne funktion sender data ved hjælp af SPI til displayet.
 * @param[in] fd er en file descripter til SPI interfacet.
 * @param[in] tx er en integer pointer til den data der skal sendes.
 * @return Retunerer 1 ved succes og ERROR ved fejl.
 */  
int sendDisp(int fd, uint8_t *tx)
{
	int ret;
	uint8_t rx[ARRAY_SIZE(tx)] = {0, };
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = ARRAY_SIZE(tx),
		.delay_usecs = (int) NULL,
		.speed_hz = (int) NULL,
		.bits_per_word = (int) NULL,
	};
	

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	
	if (!ret)
	{
		return ERROR;
	}
	
	return ret;
	
	
}


/** @brief Åbner SPI forbindlse til displayet
 * @return Retunerer en file descriptor til SPI displayet.
 */
int openDisp(void)
{
	return open(SPI_DEVICE, O_RDWR);
}

/** @brief Lukker SPI forbindelsen til displayet.
 *  @param[in] fd er en file descriptor til SPI interfacet.
 * @return Retunerer 1 ved succes og -1 ved fejl.
 */
int closeDisp(int fd)
{	
	return close(fd);
}
/*
send_data(fd, 0x0F);
send_data(fd, 0x0A);
send_data(fd, 0x0B);
send_data(fd, 0x04);
*/
