/** @package gpio.c
 * @brief Dette modul står for at oversætte knappernes nummer til kommandoer og videresende dem til netværks køen.
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <linux/input.h>
#include <fcntl.h> 

#include "queue.h"

// for testing purposes
#include <unistd.h>



#include "gpio.h"

#define EVENT_INTERFACE "/dev/input/event0"

#define BUTTON_1 KEY_1
#define BUTTON_2 KEY_2
#define BUTTON_3 KEY_3
#define BUTTON_4 KEY_4
#define BUTTON_5 KEY_5
#define BUTTON_6 KEY_6

/**
 @brief Funktionen åbner event interfacet til kernemodulet der registrerer interrupts fra knapperne.
 @return Denne funktion retunerer en file descriptor til det åbne event interface eller -1 ved fejl.
 */
int openGPIOS(void)
{
	return open( EVENT_INTERFACE ,O_RDONLY);
}

/**
 @brief Denne funktion læser fra event interfaceet og oversætter knappens værdi til en kommando der sendes videre ved hjælp af netværks køen. 
 Ved sendes en fejlbesked til display køen.
 @param[in] fd er en file descriptor til et event interface.
 @param[in] queueNetOut er en file descriptor til netværks besked køen.
 @param[in] queueDisp er en file descriptor til display besked køen.
 @return Funktionen er en evig løkke der venter på input og returnerer aldrig.
 */
int readGPIOS(int fd, int queueNetOut, int queueDisp)
{	
	msgbuf msg;
	msg.mtype = TYPE_1;
	strcpy(msg.message,"play\\05");
	/*
	while(1)
	{
		//printf("GPIO: write queue, %s %d\n", msg.message, queueNetOut);
		writeQueue(queueNetOut, &msg);	
		//printf("GPIO: write queueAfter write , %s %d\n", msg.message, queueNetOut);
		sleep(10);
	}
	*/
	
	int nBytes;
	struct input_event ev;
	
	while (1)
	{
		nBytes = read( fd, (char *)&ev, sizeof(struct input_event));
		if (nBytes > 0)
		{
			if (ev.type == EV_KEY)
			{
			
				if (ev.value == 1)
				{
					switch (ev.code)
					{
						case BUTTON_1:
							//printf("button1 pressed\n");
							strcpy(msg.message,"play\\06");
							writeQueue(queueNetOut, &msg);						
							break;
						case BUTTON_2:
							//printf("button2 pressed\n");
							strcpy(msg.message,"toggle\\05");
							writeQueue(queueNetOut, &msg);		
							break;
						case BUTTON_3:
							//printf("button3 pressed\n");
							strcpy(msg.message,"getVerses\\05");
							writeQueue(queueNetOut, &msg);		
							break;
						case BUTTON_4:
							//printf("button4 pressed\n");
							strcpy(msg.message,"getSongs\\05");
							writeQueue(queueNetOut, &msg);		
							break;
						case BUTTON_5:
							//printf("button5 pressed\n");
							strcpy(msg.message,"getTitle\\05");
							writeQueue(queueNetOut, &msg);		
							break;
						case BUTTON_6:
							//printf("button6 pressed\n");
							strcpy(msg.message,"stop");
							writeQueue(queueNetOut, &msg);		
							break;
					}
				}
			}
		}
	}
}


/** @brief Lukker event interfacet til file descriptoren i argumentet.
 * @param[in] fd er en file descriptor til event interfacet
 * @return Retunerer 0 ved succes og -1 ved fejl.
 */
int closeGPIOS(int fd)
{
	return close(fd);
}
