/** @package main.c
 * @version 0.1
 * @author Jesper Vestergaard
 * @brief Dette er klient programmet. Dette program står for at registrere inputs fra knapper og videresende dem som kommandoer til værten.
 * Respons fra værten bliver vist på displayet der er koblet til klienten.
 */
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>

#include "network.h"
#include "disp.h"
#include "queue.h"
#include "gpio.h"

void *gpioFunction(void *arguments);
void *netOutFunction(void *arguments);
void *netInFunction(void *arguments);
void *dispFunction(void *arguments);


/**
 * Hoved programmet der initialiserer de fire tråde med netværk ind, netværk ud, display og input fra knapperne.
 */
int main(void)
{
	pthread_t gpioThread, netOutThread, netInThread, dispThread;
	int sock;

	sock = initSocket();

	if ( sock == ERROR)
	{
		// Write error to stdin and display
		printf("Error: Could not open network socket\n");
		msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_SOCKET};
		exit(1);
		//writeQueue(->dispQueue, &msg);	
	}	


	pthread_create( &gpioThread, NULL, gpioFunction, (void*) NULL);
	pthread_create( &netOutThread, NULL, netOutFunction, (void*) &sock);
	pthread_create( &netInThread, NULL, netInFunction, (void*) &sock);
	pthread_create( &dispThread, NULL, dispFunction, (void*) NULL);
	
	while (1)
	{
		//printf("");
		//asm volatile (“nop; ret r12”);
		//asm volatile (“nop”);
		sleep(100);
	}


	//closeSocket(argumentsInt[NET_SOCKET]);	// close bliver aldrig kørt pga while 1 TBD: kør closeSocket ved nedlukning af program	
	return 0;
}

/**
 * Denne funktion er input tråden. Tråden kommunikerer med display tråden og netværk ud tråden ved hjælp af to besked køer.
 * Denne funktion åbner forbindelsen til de to køer samt forbindelsen til input knappernes kerne interface. Efter det ventes der på input fra knapperne.
 */
void *gpioFunction(void *arguments)
{
	int fd;
	int dispQueue, netQueue;
	
	dispQueue = createQueue(QUEUE_DISP);
	netQueue = createQueue(QUEUE_NET);
	if (netQueue < 0 || dispQueue < 0)
	{
		// Write error to stdin and display
		printf("Error: Could not create queues\n");
		//msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_QUEUE};
		//writeQueue(dispQueue, &msg);
	}
	

	fd = openGPIOS();
	if (fd < 0)
	{
		// Write error to stdin and display
		printf("Error: Could not open GPIOS\n");
		msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_GPIOS};
		//writeQueue(dispQueue, &msg);
	}
		
	readGPIOS(fd, netQueue, dispQueue); // this is a while 1 function and it will never return TBD: make it return so that the event will get closed properly
	
	closeGPIOS(fd);
	
	return 0;

}

/**
 * Funktionen er netværk ud tråden. Denne tråd åbner de to køer til displayet og netværket, lytter til netværks køen og videresender alt data fra køen over netværket til værten.
 */
void *netOutFunction(void *arguments)
{
	int * args = (int *) arguments;	
	int dispQueue, netQueue;
	msgbuf msg_recv;
	int sock;
	
	sock = *args;
	

	dispQueue = createQueue(QUEUE_DISP);
	netQueue = createQueue(QUEUE_NET);
	if (netQueue < 0 || dispQueue < 0)
	{
		// Write error to stdin and display
		printf("Error: Could not create queues\n");
		//msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_QUEUE};
		//writeQueue(dispQueue, &msg);
	}
	
	while(1)
	{
		//printf("netOUT Socket: %d\n", sock);
		readQueue(netQueue, &msg_recv, TYPE_1);
		sendSocket(sock, msg_recv.message);
		//printf("netOUT Socket: %d\n", args->netSocket);
	}

}

/**
 * Lytter til netværket og videresender alt data til display køen.
 */
void *netInFunction(void *arguments)
{
	int * args = (int *) arguments;	
	int dispQueue, netQueue;
	msgbuf msg;
	msg.mtype = TYPE_1;
	
	dispQueue = createQueue(QUEUE_DISP);
	netQueue = createQueue(QUEUE_NET);
	if (netQueue < 0 || dispQueue < 0)
	{
		// Write error to stdin and display
		printf("Error: Could not create queues\n");
		//msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_QUEUE};
		//writeQueue(dispQueue, &msg);
	}

	while(1)
	{
		recvSocket(*args, msg.message);
		//printf("netIN: %s\n", msg.message);
		writeQueue(dispQueue, &msg);	
	}
	
}

/**
 * Tråden lytter til display køen og sender data sendt til køen ud til dispayet. 
 */
void *dispFunction(void *arguments)
{
	int fd, i;
	int dispQueue;
	uint8_t digits[4];
	msgbuf msgRecv;
	
	fd = openDisp();
	if (fd == ERROR)
	{
		// Write error to stdin and display
		printf("Error: Could not open SPI disp\n");
		//msgbuf msgRecv = {TYPE_1, DISP_ERROR_OPEN_DISP};
		//writeQueue(arguments[QUEUE_DISP_OUT], &msgRecv);	
	}
	
	dispQueue = createQueue(QUEUE_DISP);
	if (dispQueue < 0)
	{
		// Write error to stdin and display
		printf("Error: Could not create queues\n");
		//msgbuf msg = {TYPE_1, DISP_ERROR_OPEN_QUEUE};
		//writeQueue(dispQueue, &msg);
	}	
	
	while(1)
	{
		readQueue(dispQueue, &msgRecv, TYPE_1);
		
		for(i = 0; i < 4; i++)
		{
			digits[i] = (int) msgRecv.message[i];
		}
		sendDisp(fd, digits);
		//printf("written %s\n", msgRecv.message);
	}
	
	closeDisp(fd);
	
}
/*

msgbuf msg = {1, "Jesper er kongen"};
msgbuf msg_recv;



queue = createQueue();
printf("created\n");
writeQueue(queue, &msg);
printf("written\n");
readQueue(queue, &msg_recv, 1);

printf("message: %s\n", msg_recv.message);


printf("queue %d\n", queue);

closeQueue(queue);
*/

/*
	int sock;
	char messageIN[MESSAGE_SIZE];
	char messageOUT[MESSAGE_SIZE];
	char character;
	
	while (1)
	{
		character = getchar(); 
		switch (character)
		{
			case 'p':
				strcpy(messageOUT,"toggle\\04");
				printf("Pause\n");
				break; 
			case 'o':
				strcpy(messageOUT,"play\\05");
				printf("Play\n");
				break;
			case 'i':
				strcpy(messageOUT,"getSongs\\05");
				printf("Get songs\n");
				break;
			case 'u':
				strcpy(messageOUT,"getTitle\\05");
				printf("Get Title\n");
				break;
			case 'y':
				strcpy(messageOUT,"getVerses\\05");
				printf("Get Verses\n");
				break;
			case 't':
				strcpy(messageOUT,"ping\\05");
				printf("Ping\n");
				break;
			case 'r':
				strcpy(messageOUT,"error\\05");
				printf("Error\n");
				break;
		}
		printf("char: %d\n", character);
		if (character != 10)
		{	 
			sock = initSocket();	
			sendSocket(sock, messageOUT);
			recvSocket(sock, messageIN);
			closeSocket(sock);		
			printf("\nmessage %s\n", messageIN);
		}
		
*/

/*
		uint8_t cipher[4];
		int i;
		for(i = 0; i < 4; i++)
		{
			cipher[i] = (int) message[i];
			printf("ciffer: %d\n", cipher[i]);
		}
*/
		//uint8_t dispdata[] = { 0x78, 0x78, 0x78, 0x78 };

		
		/* int fd = openDisp();
		sendDisp(fd, cipher);
		closeDisp(fd); 
	}
*/
		
//}


 


