/** @package network.c
 * @brief Dette module står for kommunikationen over netværket. Det åbner en socket til værten, sender data, modtager data og lukkker forbindelsen til værten.
 */
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include <unistd.h>

#include "network.h"

/** @brief Denne funktion opretter en tcp/ip netværks forbindelse til værten defineret som SERVER_IP på porten PORT.
 * @return Retunerer file descriptoren til socketen eller ERROR ved fejl.
 */
int initSocket(void)
{
	int sock;
	struct sockaddr_in server;
	
	// Create socket
	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
	{
		printf("Failure creating socket");
		printf("Sock: %d\n", sock);
		return ERROR;
	}
	
	// Create the server sockaddr_in structure
	memset(&server, 0, sizeof(server));       	// Clear struct
	server.sin_family = AF_INET;                  	// IP protokol
	server.sin_addr.s_addr = inet_addr(SERVER_IP);  // IP address
	server.sin_port = htons(PORT);					// server port
	
	// Establish connection
	if (connect(sock,(struct sockaddr *) &server,sizeof(server)) < 0)
	{
		printf("Failure connecting to server\n");
		printf("Sock: %d\n", sock);
		printf("Server IP: %s\n", SERVER_IP);
		return ERROR;
	}
	return sock;
}

/** @brief Sender en besked over tcp/ip til værten.
 * @param[in] sock er en file descriptor til socket'en.
 * @param[in] message er en char pointer til beskeden der skal sendes.
 * @return Retunerer længden på den afsendte besked eller ERROR ved fejl.
 */
int sendSocket(int sock, char* message)
{
	//printf("messageOUT: %s, sock %d\n", message, sock);
	if (send(sock, message, strlen(message), 0) != strlen(message))
	{
		printf("Failure sending message, Mis match in sent length");
		return ERROR;
	}
	return strlen(message);
}


/** @brief Modtager indgående data fra værten.
 * @param[in] sock er en file descriptor til socket'en.
 * @param[out] message er en char pointer hvor den modtagne besked placeres.
 * @return Retunerer antal modtagne bytes ved succe eller ERROR ved fejl. 
 */
int recvSocket(int sock, char* message)
{
	int bytes;
	bytes = recv(sock, message, MESSAGE_SIZE-1, 0);
	if (bytes == 0)
	{
		printf("no message recieved");
		return ERROR;
	}
	message[bytes] = '\0';
	return bytes;
}

/** @brief Lukker for forbindelsen til værten.
 * @param[in] sock er en file descriptor til den åbne socket.
 * @return Retunerer 0 ved succes og -1 ved fejl.
 */
int closeSocket(int sock)
{
	return close(sock);
}
