/** @package queue.c
 * @brief Modulet opretter, skriver, læser og lukker besked køer.
 */
//#include <sys/types.h>
//#include <sys/ipc.h>
#include <stdlib.h>
#include <sys/msg.h>
#include "queue.h"

#include <stdio.h>

#define ALL_RW 0666

/** @brief Opretter en kø svarende til den key i argumented number.
 * @param[in] number er en key der svarer til en bestemt kø.
 * @return Retunerer kø file descriptor ved succes og -1 ved fejl. Hvis køen allerede findes i forvejen retuneres file descriptoren til den eksisterende kø.
 */
int createQueue(int number)
{
	//     key = ftok("/dir/to/somefile", 'b'); can  used to create a derterment key
	return msgget(number, ALL_RW | IPC_CREAT);
}

/** @brief Funktionen tilføjer en besked til en kø.
 * @param[in] mQueueID er file descriptoren på en besked kø.
 * @param[in] msg er en pointer der peger på en msgbuf.
 * @return Retunerer 0 ved succes og -1 ved fejl.
 */
int writeQueue(int mQueueId, msgbuf *msg)
{
	//printf("writeQueue, %d\n", mQueueId);
	return msgsnd(mQueueId, msg, sizeof(msgbuf), 0);
}

/** @brief Denne funktion læser og fjerner den først tilkommende besked i en kø.
 * @param[in] mQueueId er en file descriptor til en besked kø.
 * @param[out] msg er en pointer til hvor den læste msgbuf skal placeres.
 * @param[in] type er hvilken gruppe beskeden der skal læses tilhører.
 * @return Retunerer 0 ved succesfuld læsning og -1 ved fejl.
 */

int readQueue(int mQueueId, msgbuf *msg, int type)
{
	//printf("readQueue, %d\n", mQueueId);
	return msgrcv(mQueueId, msg, sizeof(msgbuf), type, 0);
}

/** @brief Lukker for en kø.
 * @param[in] mQueueId er en file descriptor på den kø der skal lukkes ned.
 * @return Retunere 0 ved succesfuld nedlukning af kø og -1 ved fejl.
 */
int closeQueue(int mQueueId)
{
	return msgctl(mQueueId, IPC_RMID, NULL);
}
