# -*- coding: utf-8 -*-
## @package main.py
#  Det er værten. Dette module består af to tråde hvor den ene tråd lytter til en forudbestemt port på netværket og den anden tråd afspiller sange.
#
#  

import threading
import Queue
import time

import mad, ao, sys

# for the network handling
from twisted.internet.protocol import Protocol, Factory
from twisted.internet import protocol, reactor, defer



ERROR_OK = 0


## Denne klasse definerer regler for hvordan værten skal opføre sig ved forskellige forbindelses hændelser.
#Klassen nedarver fra twisted's generelle procolkol klasse. 
#
class JukeProtocol(Protocol):
    ## Når en forbindelse oprettes skrives der forbindelse oprettet i loggen samt et tidsstempel
    def connectionMade(self):
        print "connection established"
       
    ## Når en forbindelse oprettes skrives der forbindelse oprettet i loggen samt et tidsstempel
    def connectionLost(self, reason):
	print "connection lost"
	#sys.exit(main())

    ## Denne funktion modtager data fra netværket og sender det videre til factory klassen som processerer det.
    # Processerigen foregår asynkront og efter endt processering bliver status sendt tilbage til klienten.
    def dataReceived(self, data):
	d = self.factory.processData(data)
	print d
	
        #def onError(err):
        #    return 'Internal error in server'
        #d.addErrback(onError)

        def writeResponse(message):
            self.transport.write(message)
            #self.transport.loseConnection()
        d.addCallback(writeResponse)





## Denne klasse er en wrapper klasse som sørger for at processeringen af netværks data bliver gjort asynkront.
class JukeFactory(protocol.ServerFactory):
    protocol = JukeProtocol

    def __init__(self, application):
        self.application = application
    ## Funktionen sender processeringsdata videre.
    def processData(self, data):
	return defer.succeed(self.application.processCommand(data))



## Denne klasse står for selve processeringen af netværks hændelser.
# 
class Application():
	def __init__(self, queueApplication, queuePlayer):
		self.filelist = (("music/01.mp3", 5), # tuple array, read only
				("music/02.mp3", 4),
				("music/03.mp3", 3),
				("music/04.mp3", 2),
				("music/05.mp3", 1),
				("music/06.mp3", 6),
				("music/07.mp3", 7),
				("music/08.mp3", 8),
				("music/09.mp3", 9))

		self.queueApplication = queueApplication
		self.queuePlayer = queuePlayer
	## Denne funktion splitter dataen op i to dele. Den ene del er kommandoen og den anden del er et medfølgende argument.
	# Disse kommandoer bliver udført og resultatet af udførelsen bliver returnet.
	def processCommand(self, data):
		data = data.strip()
		command = data.split("\\0") # command(0) is command and command(1) is argument
		
		#Control section
		if command[0] == "play":
			try:
				return self.play(int(command[1]));
			except ValueError:
				return "E004"
			
		elif command[0] == "toggle":
			return self.toggle()
			
		elif command[0] == "stop":
			return self.stop()
		
		#Display section
		elif command[0] == "getSongs":
			return self.getSongs()
		elif command[0] == "getTitle":
			return self.getTitle(int(command[1]))
		elif command[0] == "getVerses":
			return self.getVerses(int(command[1]))
		#Keepalive
		elif command[0] == "ping":
			return self.ping()
		elif command[0] == "error":
			return self.error()
			
	## Funktionen starter sender signal til Player tråden om at starte afspilningen af filen specificeret i file_id.
	def play(self, file_id):
		self.queuePlayer.put("play")
		self.queuePlayer.put(self.filelist[file_id][0])
		return self.queueApplication.get()
	## Toggle sender signal til Player tråden om at toggle den nuværende sang.
	def toggle(self):
		self.queuePlayer.put("toggle")
		return self.queueApplication.get()
	## Denne funktion sender signal til Player tråden om at stoppe den nuværende sang.
	def stop(self):
		self.queuePlayer.put("toggle")
		self.queueApplication.get()
		return "9999"
	## Retunerer antallet af sange tilgængelige.
	def getSongs(self):
		str(len(self.filelist))
		return "8888"
	## Funktionen returnerer fil stien på sangen tilhørende fil id'et index.
	def getTitle(self, index):
		self.filelist[index][0]
		return "7777"
	## Retunerer antallet af vers i sangen med fil id'et index
	def getVerses(self, index):
		str(self.filelist[index][1])
		return "6666"
	## Retunerer "pong" til klienten
	def ping(self):
		return "pong"
	## Hvis der opstår en fejl på klienten vil fejlkoden blive sendt til værten og skrevet til loggen.
	def error(self):
		print "error"
		return "error"

		

		

		
		



## Denne klasse står for afspilningen af musiknumre. Den er en tråd og kommunikerer med Application klassen over køer.
class Player(threading.Thread):
	## Initieringen af klassen. Her bliver de to køer der bruges til at kommunikere med Application klassen sat.
	def __init__(self, queuePlayer, queueApplication):
		self.queueApplication = queueApplication
		self.queuePlayer = queuePlayer
		self.mf = mad.MadFile("empty.mp3")
		self.mfTemp = self.mf 
		self.isPaused = False
		print "init player"
		threading.Thread.__init__(self)
	## Den evige løkke hvor play funktionen bliver kaldt igen og igen.
	def run(self):
		while True:
			self.play()
			time.sleep(0.01)
			
	## Afspiller funktion der afspiller filen i variablen file.
	def play(self):
		#self.mf = mad.MadFile(file)
		
		while True:
			result = self.checkQueue()
			#print self.isPaused
			if result == 'toggle':
				if self.isPaused:
					self.mf = self.mfTemp
					self.isPaused = False
				else:
					self.mfTemp = self.mf
					self.mf = mad.MadFile("empty.mp3")
					self.isPaused = True				
				self.queueApplication.put("2222")
				#break
				
			elif result == 'play':
				file = self.queuePlayer.get() # after a play the new filename is in the queue
				print file
				self.mf = mad.MadFile(file)	
				self.dev = ao.AudioDevice('alsa', rate=self.mf.samplerate())
				self.isPaused = False
				self.queueApplication.put("1111")
			
			self.buf = self.mf.read()
			if self.buf is None:
				break				
			self.dev.play(self.buf, len(self.buf))
	## Funktion der bliver brugt at play funktionen, under afspilning, til at kontrollere om der er kommet ny data i en afspiller køen.
	def checkQueue(self):
		if (not self.queuePlayer.empty()):
			try:
				return self.queuePlayer.get_nowait()
			except Queue.Empty:
				return False
		return False


## Hoved funktionen der starter de to tråde med Player klassen og netværks lytteren.
def main():
	queuePlayer = Queue.Queue(0)
	queueApplication = Queue.Queue(0)
	
	
	app = Application(queueApplication, queuePlayer)
	factory = JukeFactory(app)
	
	Player(queuePlayer, queueApplication).start()
	
	reactor.listenTCP(50001, factory)
	reactor.run()
	



if __name__ == "__main__":
    main()

